#include <stdlib.h>
#include "mandelbrot.h"

unsigned short max_iter=64;
extern RGB palette[];


static double _cabs(struct _complex z)
{
  return sqrt( z.x *z.x + z.y * z.y); 
}

static struct _complex square( const struct _complex c)
{
	struct _complex r;

	r.x = c.x * c.x - c.y * c.y;
	r.y =  c.x * c.y * 2;
	return r;
}

static int iterate( struct _complex c)
{
	int i;
	struct _complex z;


	for(z=c, i=0; _cabs(z) < 2.0 && i < max_iter; i++)
	{
		//viva o c++!

		z = square(z);
		z.x += c.x;
		z.y += c.y;
	}
	return i/2;
}

void compute(MANDELBROT * self)
{
	unsigned int linha, coluna, nlinhas , ncolunas;
	double deltax;
	struct _complex c;
	BMP *bmp;

	deltax = self->reg.end.x - self->reg.start.x;
	ncolunas = self->resolution;
	nlinhas = (unsigned int) ((self->reg.end.y - self->reg.start.y)/deltax * self->resolution);

	printf("linhas = %d colunas = %d\n", nlinhas, ncolunas);

	bmp = bmpCreate(nlinhas, ncolunas);
	
	c=self->reg.start;

	for(linha=0; linha < nlinhas;  linha++ )
	{
		RGB  *linhaBMP = getLineStart(bmp, linha);

		for(coluna=0; coluna <  ncolunas; coluna++)
		{
			
			int iteracoes;

			iteracoes = iterate(c);

			linhaBMP[coluna] = palette[iteracoes];
			c.x = deltax * coluna / self->resolution + self->reg.start.x;
		}

		c.y = deltax * linha / self->resolution + self->reg.start.y;
	}

	bmpSave(bmp, self->filename);
	free(bmp);

}
